import bpy
import os
import subprocess as sp

from .Functions import AssettoCorsa, TrackFolder, get_properties, find_ac_installation, read_json


from pathlib import Path
class PROJECT_OT_export_fbx_for_ac(bpy.types.Operator):
    bl_idname = "project.export_fbx_for_ac"
    bl_label = "Export FBX for AC"
    bl_description = """Export scene to FBX with Assetto-Corsa-Standards. You still can export manually if problems occur.
Parameters:
            filepath=fbx_path, 
            check_existing=False, 
            global_scale=0.01,
            apply_unit_scale=True,
            apply_scale_options='FBX_SCALE_ALL',
            object_types={'EMPTY', 'MESH', 'OTHER'},
            axis_forward='-Z', 
            axis_up='Y',
            use_tspace=True,
            use_mesh_modifiers=True,
            use_mesh_modifiers_render=True,
            use_triangles=True,
            mesh_smooth_type='OFF',
            colors_type='SRGB'
            """

    def execute(self, context):
        props = get_properties(context)
        fbx_path = os.path.join(os.path.dirname(bpy.data.filepath), Path(bpy.data.filepath).stem + ".fbx")

        # EXPORT SCENE as FBX
        bpy.ops.export_scene.fbx(
            filepath=fbx_path, 
            check_existing=False, 
            global_scale=0.01, # if problems -> set to 1
            apply_unit_scale=True,
            apply_scale_options='FBX_SCALE_ALL',
            object_types={'EMPTY', 'MESH', 'OTHER'},
            axis_forward='-Z', 
            axis_up='Y',
            use_tspace=True,
            use_mesh_modifiers=True,
            use_mesh_modifiers_render=True,
            use_triangles=True,
            mesh_smooth_type='OFF',
            colors_type='SRGB'
            )
        
        self.report({'INFO'}, f"Exported to {fbx_path}")
        return {'FINISHED'}


class PROJECT_OT_export_track_folder(bpy.types.Operator):
    bl_idname = "project.export_track_folder"
    bl_label = "Export Track to AC"
    bl_description = "Export Track to AC (overwrite existing files) - You need to convert the fbx in KsEditor first!"
    
    def execute(self, context):
        import shutil

        props = get_properties(context)

        if not props.disable_export_checks and not ready_for_export(self, context):
            return {'FINISHED'}

        tf = TrackFolder(props.track_folder)
        ac = AssettoCorsa(props.ac_installation)

        abs_track_folder =tf._path
        if not abs_track_folder:
            self.report({'INFO'}, f"Something went wrong! Cannot locate '{abs_track_folder}'")
            return {'FINISHED'}

        if not track_is_valid(self, abs_track_folder):
            return {'FINISHED'}
        
        dest = os.path.join(ac._content_tracks, tf._track_name)

        shutil.copytree(src=abs_track_folder, dst=dest, dirs_exist_ok=True) # copy

        self.report({'INFO'}, f"Copied Track Folder to {dest}")
        return {'FINISHED'}


class PROJECT_OT_find_ac(bpy.types.Operator):
    bl_idname = "project.ks_finder"
    bl_label = "Find AC Installation"
    bl_description = "Find your local installation of Assetto Corsa"

    def execute(self, context):
        props = get_properties(context)

        props.ac_installation = os.path.normpath(find_ac_installation())
        if not props.ac_installation:
            self.report({'INFO'}, "Could not locate AC Installation.")
        return {'FINISHED'}


class PROJECT_OT_start_kseditor(bpy.types.Operator):
    bl_idname = "project.ks_editor"
    bl_label = "KsEditor"
    bl_description = "Open KsEditor (located in your AC installation -> ./sdk/editor/ksEditor.exe)"
    
    def execute(self, context):
        props = get_properties(context)

        if not os.path.exists(props.ac_installation):
            props.ac_installation = os.path.normpath(find_ac_installation())
        ks_editor = os.path.normpath(os.path.join(props.ac_installation, "sdk\\editor\\KsEditor.exe"))

        if not ks_editor:
            self.report({'INFO'}, "Path for exe missing!")
            return {'FINISHED'}
        
        if not os.path.exists(ks_editor):
            self.report({'INFO'}, f"Cannot find KsEditor in path '{ks_editor}'")
            return {'FINISHED'}

        #open_folder(ks_editor)
        import subprocess as sp
        #fbx = os.path.join(bpy.data.filepath, os.path.basename(bpy.data.filepath) + '.fbx')
        p = sp.Popen([ks_editor], cwd=os.path.dirname(ks_editor))
        """
        p.wait() # wait only in background & popup window for export?
        self.report({'INFO'}, f"{p.pid} has finished! Welcome back in Blender!")
        """

        return {'FINISHED'}


def track_is_valid(self, folder):
    track_file = os.path.join(folder, os.path.basename(os.path.dirname(folder)) + '.kn5')

    if not os.path.exists(track_file):
        self.report({'INFO'}, f"Track file '{track_file}' does not exist! Export from KsEditor first.")
        return False
    
    return True


# Additional/Functional Export Checks (can be disabled through property 'disable_export_checks')
from .Properties import AC_OBJ_PREFIX, AC_COLLECTION_NAME, FILE_UI_TRACK
def ready_for_export(self, context: bpy.types.Context):
    if not ac_objects_valid(self, context):
        return False
    
    return True


# sub method of ready_for_export
def ac_objects_valid(self, context):
    if AC_COLLECTION_NAME not in context.view_layer.layer_collection.children:
        self.report({'INFO'}, f"Collection '{AC_COLLECTION_NAME}' doesn't exist. Please add AC-Objects before export!")
        return False
    
    objects = context.view_layer.layer_collection.children[AC_COLLECTION_NAME].collection.objects
    
    # check if AC objects exist and match track_ui pits etc
    for prefix in AC_OBJ_PREFIX.values():
        # check if existing at all
        if not any(obj.name.startswith(prefix) for obj in objects):
            self.report({'INFO'}, f"AC-Object missing! '{prefix}_<n>'")
            return False
        
        # check timing
        if prefix == AC_OBJ_PREFIX['TIME']:
            count = sum(1 for obj in objects if obj.name.startswith(prefix))
            if count % 2 > 0:
                self.report({'INFO'}, f"One timing object is missing!")
                return False
            continue

        # check pits
        if prefix == AC_OBJ_PREFIX['PIT']:
            try:
                props = get_properties(context)
                data = read_json(TrackFolder(props.track_folder).get_ac_file_path(FILE_UI_TRACK))
                required_pit_count = int(data['pitboxes'])
            except:
                self.report({'INFO'}, f"Could not extract pit count from {FILE_UI_TRACK}. Please check manually!")
                return False
            
            scene_pit_count = sum(1 for obj in objects if obj.name.startswith(prefix))
            if scene_pit_count != required_pit_count:
                self.report({'INFO'}, f"Required pit count: '{required_pit_count}' - pits in scene: '{scene_pit_count}'. Setup pits or adjust pit setting in track ui file!")
                return False
            continue
        
    return True
